//########## Cursed Skeletons - Custom Zombie Example

// This is an example to explain how you can add your own custom enemies.

// There are zombie types to allow one to enable/disable certain types of
// zombies in the Slayer menu.

// To add a new zombie type the below line is required.

Slayer.Prefs.addPref("EZM","Allow Cursed Skeletons","%mini.EZM_AllowCursedSkeletons","bool",1,0,1,-1,"Rules EZM Mode");

// The first parameter must always be "EZM" or the option won't be added to this gamemode.
// The second parameter defines what the option reads in the Slayer menu.
// The third is the property which saves if this type is enabled or disabled.
// The rest isn't really anything you need to look into.

// The below line is used to add a new enemy.

// EZM_addZombie(name,ingame_name,playerdatablock,scale,list_meleeweaponimages,list_rangedweaponimages,has_specialwave,spawn_propability,zombietype);

// For bosses it's very similar.

// EZM_addZombie(name,ingame_name,playerdatablock,scale,list_meleeweaponimages,list_rangedweaponimages,spawn_propability,zombietype);

// Zombies can have multiple appearances that are randomly selected.
// Normal enemies and bosses use the same function.

// EZM_addAppearance(name,nodes,hidenodes,facedecal,chestdecal,mountedimage)

// Note that the appearance will be added to the enemy with the matching name.
// "nodes" is a list of nodes and their colors that are visible on the enemy.
// If this node is hidden by default it will automatically be made visible.
// First the nodename, then it's color. Example: "headskin 1 0.5 0 1 chest 0 1 0 1".
// Note that you must add values for red, green, blue and alpha or else it will break!
// "nidenodes" is a list of nodes that will be hidden. Colors are not required.
// "mountedimage" lets you mount any image to the enemies slot 2.

// If you have questions send me (Gravity Cat) a PM on the Blockland Forum.

// Here's the example:

//### Effects

datablock ParticleData(EZM_CursedSkeletonSmokeParticle)
{
  dragCoefficient = 4;
  gravityCoefficient = -2;
  inheritedVelFactor = 0.5;
  constantAcceleration = 0;
  lifetimeMS = 1000;
  lifetimeVarianceMS = 500;
  textureName = "base/data/particles/cloud";
  spinSpeed = 10;
  spinRandomMin = -50;
  spinRandomMax = 50;
  colors[0] = "0 0 0 0";
  colors[1] = "0 1 0 0.5";
  colors[2] = "0 0 0 0";
  sizes[0] = 1;
  sizes[1] = 2;
  sizes[2] = 1;
  times[0] = 0;
  times[1] = 0.1;
  times[2] = 1;
  useInvAlpha = true;
};

datablock ParticleEmitterData(EZM_CursedSkeletonSmokeEmitter)
{
  uiName = "";
  ejectionPeriodMS = 100;
  periodVarianceMS = 0;
  ejectionVelocity = 1;
  velocityVariance = 1;
  ejectionOffset = 0.5;
  thetaMin = 0;
  thetaMax = 180;
  phiReferenceVel = 0;
  phiVariance = 360;
  overrideAdvance = false;
  particles = "EZM_CursedSkeletonSmokeParticle";
};

//### Images

datablock ShapeBaseImageData(EZM_RibcageDarkImage)
{
  shapeFile = "./ribcage.dts";
  emap = true;
  mountPoint = 7;
  offset = "0 0 0.62";
  eyeOffset = "0 0 10";
  rotation = eulerToMatrix("0 0 0");
  scale = "1 1 1";
  correctMuzzleVector = true;
  doColorShift = true;
  colorShiftColor = "0.1 0.1 0.1 1";

  stateName[0] = "Smoke";
  stateTransitionOnTimeout[0] = "Smoke";
  stateWaitForTimeout[0] = true;
  stateTimeoutValue[0] = 1000;
  stateEmitter[0] = EZM_CursedSkeletonSmokeEmitter;
  stateEmitterTime[0] = 1000;
};

datablock ShapeBaseImageData(EZM_CrownImage)
{
  shapeFile = "./items/crown.dts";
  emap = true;
  mountPoint = $HeadSlot;
  offset = "0 0 -0.015";
  eyeOffset = "0 -0.19 0";
  rotation = eulerToMatrix("0 0 0");
  scale = "1 1 1";
  correctMuzzleVector = true;
  doColorShift = true;
  colorShiftColor = "0.1 0.1 0.1 1";

  stateName[0] = "Smoke";
  stateTransitionOnTimeout[0] = "Smoke";
  stateWaitForTimeout[0] = true;
  stateTimeoutValue[0] = 1000;
  stateEmitter[0] = EZM_CursedSkeletonSmokeEmitter;
  stateEmitterTime[0] = 1000;
};

//### Player Datablocks

datablock PlayerData(EZM_CursedSkeletonPlayer : PlayerStandardArmor)
{
  uiName = "";
  maxDamage = 150;
  jumpForce = 1200;
  maxForwardSpeed = 10;
  maxBackwardSpeed = 6;
  maxSideSpeed = 8;
  maxForwardCrouchSpeed = 5;
  maxBackwardCrouchSpeed = 3;
  maxSideCrouchSpeed = 3;
  canJet = false;
};

datablock PlayerData(EZM_CursedKnightPlayer : PlayerStandardArmor)
{
  uiName = "";
  maxDamage = 500;
  jumpForce = 1200;
  canJet = false;
};

datablock PlayerData(EZM_CursedHorsePlayer : HorseArmor)
{
  className = Armor;
  maxDamage = 300;
  uiName = "";
};

function EZM_CursedHorsePlayer::onAdd(%this,%obj)
{
  parent::onAdd(%this,%obj);
  %mini = %obj.minigame;
  %i = 0;
  while($EZM_Zombies[%i,"Name"] !$= "")
  {
    if($EZM_Zombies[%i,"Name"] $= "Cursed Knight") break;
    %i += 1;
  }
  %name = $EZM_Zombies[%i,"Name"];
  %namelist = $EZM_Zombies[%i,"Namelist"];
  %db = $EZM_Zombies[%i,"Datablock"];
  %playersize = $EZM_Zombies[%i,"PlayerSize"];
  %melee = $EZM_Zombies[%i,"Melee"];
  %ranged = $EZM_Zombies[%i,"Ranged"];
  %intelligence = mClampF(%mini.EZM_ZombieIntelligence + %mini.EZM_ZombieIntelligenceInc,0,100);
  if(%db $= "") return;
  %obj.bot = new AIPlayer()
  {
    dataBlock = %db;
    position = %obj.getPosition();
    EZMisZombie = true;
    home = %obj.home;
    intelligence = %intelligence;
    sightRange = 50 + (%intelligence * 4.5);
    attackRange = 5;
    minigame = %mini;
    name = %namelist;
  };
  if(isObject(%obj.bot))
  {
    %obj.bot.setPlayerScale(%playersize);
    %obj.bot.spawnexplosion(SpawnProjectile,%obj.bot.getScale());
    %mini.EZM_ZombieCount += 1;
    %mini.EZM_ZombieWaveCount -= 1;
    %obj.bot.EZM_Idle();
    %obj.bot.client = %obj.bot;
    EZMZombies.add(%obj.bot);
    %app = $EZM_ZombieAppearances[%name,0,"Appearance"];
    %hide = $EZM_ZombieAppearances[%name,0,"HideNodes"];
    %face = $EZM_ZombieAppearances[%name,0,"FaceDecal"];
    %chest = $EZM_ZombieAppearances[%name,0,"ChestDecal"];
    %image = $EZM_ZombieAppearances[%name,0,"MountedImage"];
    if(%app != -1) {
      for(%i=0;%i<getWordCount(%app);%i++) {
        %obj.bot.unHideNode(getWord(%app,%i));
        %obj.bot.setNodeColor(getWord(%app,%i),getWords(%app,%i + 1,%i + 4));
        %i += 4; }
    }
    if(%hide != -1) { for(%i=0;%i<getWordCount(%hide);%i++) { %obj.bot.hideNode(getWord(%hide,%i)); } }
    %obj.bot.setFaceName(getWord(%face,getRandom(getWordCount(%face)-1)));
    %obj.bot.setDecalName(getWord(%chest,getRandom(getWordCount(%chest)-1)));
    if(%image != -1) %obj.bot.mountImage(%image,2);
    %rnd = 0;
    %obj.bot.playThread(0,"armReadyRight");
    if(%mini.EZM_ZombieWeapons == 0)
      %weapon = 1;
    else if(%mini.EZM_ZombieWeapons == 1)
      %weapon = 2;
    else
    {
      if(getRandom(3)) %weapon = 1;
      else
        %weapon = 2;
    }
    if(%ranged == -1) %weapon = 1;
    if(%melee == -1) %weapon = 2;
    if(%weapon == 1) {
      %rnd = getRandom(0,getWordCount(%melee) - 1);
      %obj.bot.mountImage(getWord(%melee,%rnd),0); }
    else {
      %rnd = getRandom(0,getWordCount(%ranged) - 1);
      %obj.bot.mountImage(getWord(%ranged,%rnd),0);
      %obj.bot.attackRange = %obj.bot.sightRange / 2; }
    %obj.mountObject(%obj.bot,2);
  }
}

//### Enemies

EZM_addZombie("Cursed Knight","Cursed Knight",EZM_CursedKnightPlayer,"1 1 1","EZM_SwordImage","EZM_BowImage",1,2,"EZM_AllowCursedSkeletons");
EZM_addAppearance("Cursed Knight","headskin 0.1 0.1 0.1 1 lhand 0.1 0.1 0.1 1 rhand 0.1 0.1 0.1 1 larmslim 0.1 0.1 0.1 1 rarmslim 0.1 0.1 0.1 1 lshoe 0.1 0.1 0.1 1 rshoe 0.1 0.1 0.1 1 pointyhelmet 0.2 0.2 0.2 1 shoulderpads 0.2 0.2 0.2 1","larm rarm chest pants","Skull","AAA-None",EZM_RibcageDarkImage);

EZM_addZombie("Cursed Mage","Cursed Mage",EZM_CursedSkeletonPlayer,"1 1 1",-1,"EZM_DeathWandImage",1,4,"EZM_AllowCursedSkeletons");
EZM_addAppearance("Cursed Mage","headskin 0.1 0.1 0.1 1 lhand 0.1 0.1 0.1 1 rhand 0.1 0.1 0.1 1 larmslim 0.1 0.1 0.1 1 rarmslim 0.1 0.1 0.1 1 lshoe 0.1 0.1 0.1 1 rshoe 0.1 0.1 0.1 1 cape 0 0.2 0 1","larm rarm chest pants","Skull","AAA-None",EZM_RibcageDarkImage);

EZM_addZombie("Cursed Horse","Cursed Horse",EZM_CursedHorsePlayer,"1 1 1","EZM_HorseWeaponImage",-1,1,2,"EZM_AllowCursedSkeletons");
EZM_addAppearance("Cursed Horse","ALL 0.15 0.2 0.1 1","","Skull","AAA-None");
EZM_addAppearance("Cursed Horse","ALL 0.22 0.2 0.15 1","","Skull","AAA-None");

//### Boss

EZM_addBoss("Sir Killaton","Sir Killaton",EZM_BossPlayer,"2 2 2",-1,"EZM_BowImage",5,"EZM_AllowCursedSkeletons");
EZM_addAppearance("Sir Killaton","headskin 0.1 0.1 0.1 1 lhand 0.1 0.1 0.1 1 rhand 0.1 0.1 0.1 1 larmslim 0.1 0.1 0.1 1 rarmslim 0.1 0.1 0.1 1 lshoe 0.1 0.1 0.1 1 rshoe 0.1 0.1 0.1 1 pointyhelmet 0.2 0.2 0.2 1 shoulderpads 0.2 0.2 0.2 1","larm rarm chest pants","Skull","AAA-None",EZM_RibcageDarkImage);

EZM_addBoss("King Everpain","King Everpain",EZM_BossPlayer,"2 2 2",-1,"EZM_DeathWandImage",5,"EZM_AllowCursedSkeletons");
EZM_addAppearance("King Everpain","headskin 0.56 0.93 0.96 1 lhand 0.56 0.93 0.96 1 rhand 0.56 0.93 0.96 1 larmslim 0.1 0 0 1 rarmslim 0.1 0 0 1 chest 0.1 0 0 1 pants 0.1 0.1 0.1 1 lshoe 0.1 0.1 0.1 1 rshoe 0.1 0.1 0.1 1 epaulets 0.6 0.6 0 1 cape 0.2 0 0.4 1","larm rarm","Skull","Knight",EZM_CrownImage);
